import torch
import torch.nn as nn


class Swapout(nn.Module):
    """
    The Swapout operator randomly swaps a pixel's features with the global average.

    Args:
        p: The probability of each pixel to be swapped.
    """

    def __init__(self, p=0.0):
        super(Swapout, self).__init__()

        if p < 0.0 or p > 1.0:
            raise ValueError('The probability of swapout must be within [0.0, 1.0]')

        self.p = p

    def forward(self, x):
        if self.training:
            # Modify the input at the training phase so that the expected outcome equals that at the testing phase
            g = torch.mean(x, dim=(2, 3), keepdim=True).expand_as(x)
            if self.p < 1.0:
                probs = torch.ones((x.size(0), 1, x.size(2), x.size(3)), dtype=x.dtype, device=x.device) * self.p
                mask = torch.bernoulli(probs).expand_as(x)
                x = mask * g + (1.0 - mask) * (x - g * self.p) / (1.0 - self.p)
            else:
                x = g
        else:
            # Keep the input as it is at the testing phase
            pass

        return x



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Swapout",
    version="0.0.1",
    author="Shanfeng Hu",
    author_email="shanfeng.hu1991@gmail.com",
    description="Swapout: Exchanging Pixels for Long-range Spatial Learning",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/shanfenghu/Swapout",
    py_modules=["Swapout"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)
